var Align = function()
{
	var align = {};
	var debug = 0;
	var subtypeMergedSequences = null;
	var clusterIsValid = false;
	
	// Page initialization
	align.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectMergedSequences', align.selectMergedSequences);
		Events.addEventHandler('mergedSequences', 'base-selected', align.setMergedSequenceCallback);
		Events.addEventHandler('step-1', 'wizard-validate', align.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', align.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', align.validateStep2);
		Events.addEventHandler('clusters', 'change', align.clusterOnChange);
		Events.addEventHandler('maskingSoftware', 'change', align.softwareOnChange);
		Events.addEventHandler('alignSoftware', 'change', align.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', align.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../MaskAlign.servlet?ID='+App.getSessionId();
		url += '&cmd=GetMergedSequencesForAlignment';
		Wizard.asyncJsonRequest(url, align.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	align.initializeStep1 = function(response)
	{
		var mergedSequences = response.mergedSequences;
		var frm = document.forms['reggie'];	

		if (mergedSequences != null && mergedSequences.length > 0)
		{
			var firstPlate = null;
			for (var msNo=0; msNo < mergedSequences.length; msNo++)
			{
				var ms = mergedSequences[msNo];
				var lib = ms.lib;
				var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				
				var name = (msNo+1) + ': ';
				if (libPlate) name += libPlate.name + ' - ';
				name += ms.name;
				if (ms.AutoProcess == 'ReProcess')
				{
					name += ' [R]';
				}
				
				var option = new Option(name, ms.id, false, libPlate != null && libPlate.name==firstPlate);
				option.mergedSequences = ms;
				frm.mergedSequences.options[frm.mergedSequences.length] = option;
			}
		}
		else
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', 'No new sequences available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.mergedSequences.focus();
	}
	
	align.validateStep1 = function(event)
	{
		Wizard.setInputStatus('mergedSequences', '', '');
		var frm = document.forms['reggie'];
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) return;
		}
		Wizard.setInputStatus('mergedSequences', 'invalid', 'Select at least one item');
		event.preventDefault();
	}
	
	align.selectMergedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.mergedSequences.disabled) return;
		
		if (subtypeMergedSequences == null)
		{
			subtypeMergedSequences = Reggie.getSubtypeInfo('MERGED_SEQUENCES');
		}
		
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeMergedSequences.id;
		Dialogs.selectItem('DERIVEDBIOASSAY', 'mergedSequences', 1, url);
	}

	align.setMergedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var frm = document.forms['reggie'];
		var mergedSequences = frm.mergedSequences;
		for (var i = 0; i < mergedSequences.length; i++)
		{
			if (mergedSequences[i].value == ms.id)
			{
				mergedSequences[i].selected = true;
				Wizard.setInputStatus('mergedSequences');
				return;
			}
		}

		var ms = event.detail;
		var option = new Option(ms.name, ms.id);
		option.mergedSequences = ms;
		option.selected = true;
		mergedSequences[mergedSequences.length] = option;
		Wizard.setInputStatus('mergedSequences');
	}

	
	// --- Step 2 -----------------------------------
	align.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('MASKING_PROTOCOL', 'maskingProtocol');
		Reggie.loadSoftware('MASKING_SOFTWARE', 'maskingSoftware', 'PARAMETER_SET');
		Reggie.loadProtocols('ALIGNMENT_PROTOCOL', 'alignProtocol');
		Reggie.loadSoftware('ALIGNMENT_SOFTWARE', 'alignSoftware', 'PARAMETER_SET');
		
		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, align.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	align.validateStep2 = function()
	{
		if (!clusterIsValid) event.preventDefault();
	}
	
	align.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.clusters;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available Open Grid clusters');
			clusterIsValid = false;
		}
		else
		{
			align.clusterOnChange();
		}
	}
	
	align.softwareOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, align.parameterSetInfoLoaded);
		}
	}
	
	align.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	align.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	

	align.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.maskingSoftware = parseInt(frm.maskingSoftware.value);
		submitInfo.maskingProtocol = parseInt(frm.maskingProtocol.value);
		submitInfo.alignSoftware = parseInt(frm.alignSoftware.value);
		submitInfo.alignProtocol = parseInt(frm.alignProtocol.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var mergedSequences = [];
		submitInfo.mergedSequences = mergedSequences;
			
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) 
			{
				var ms = {};
				ms.id = frm.mergedSequences[msNo].mergedSequences.id;
				mergedSequences[mergedSequences.length] = ms;
			}
		}

		var url = '../MaskAlign.servlet?ID='+App.getSessionId();
		url += '&cmd=StartMaskingAndAlignment';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, align.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	align.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return align;
}();

Doc.onLoad(Align.initPage);

